@@ -92,7 +92,7 @@ pip install {kwargs['project_name_normalized']}
 ## License
 
 `{kwargs['project_name_normalized']}` is distributed under the terms of the [MIT](https://spdx.org/licenses/MIT.html) license.
-""",  # noqa: E501
+""",
         ),
         File(
             Path('pyproject.toml'),
@@ -128,9 +128,9 @@ dependencies = [
 ]
 
 [project.urls]
-Documentation = "https://github.com/unknown/{kwargs['project_name_normalized']}#readme"
-Issues = "https://github.com/unknown/{kwargs['project_name_normalized']}/issues"
-Source = "https://github.com/unknown/{kwargs['project_name_normalized']}"
+Documentation = "https://github.com/{kwargs['author']}/{kwargs['project_name_normalized']}#readme"
+Issues = "https://github.com/{kwargs['author']}/{kwargs['project_name_normalized']}/issues"
+Source = "https://github.com/{kwargs['author']}/{kwargs['project_name_normalized']}"
 
 [project.scripts]
 {kwargs['project_name_normalized']} = "{kwargs['package_name']}.cli:{kwargs['package_name']}"
@@ -138,28 +138,8 @@ Source = "https://github.com/unknown/{kwargs['project_name_normalized']}"
 [tool.hatch.version]
 path = "src/{kwargs['package_name']}/__about__.py"
 
-[tool.hatch.envs.default]
-dependencies = [
-  "coverage[toml]>=6.5",
-  "pytest",
-]
-[tool.hatch.envs.default.scripts]
-test = "pytest {{args:tests}}"
-test-cov = "coverage run -m pytest {{args:tests}}"
-cov-report = [
-  "- coverage combine",
-  "coverage report",
-]
-cov = [
-  "test-cov",
-  "cov-report",
-]
-
-[[tool.hatch.envs.all.matrix]]
-python = ["3.8", "3.9", "3.10", "3.11", "3.12"]
-
 [tool.hatch.envs.types]
-dependencies = [
+extra-dependencies = [
   "mypy>=1.0.0",
 ]
 [tool.hatch.envs.types.scripts]
@@ -183,6 +163,6 @@ exclude_lines = [
   "if __name__ == .__main__.:",
   "if TYPE_CHECKING:",
 ]
-""",  # noqa: E501
+""",
         ),
     ]
