@@ -2,7 +2,7 @@ from hatch.template import File
 from hatch.utils.fs import Path
 from hatchling.metadata.spec import DEFAULT_METADATA_VERSION
 
-from ..new.default import get_files as get_template_files
+from ..new.feature_no_src_layout import get_files as get_template_files
 
 
 def get_files(**kwargs):
@@ -11,7 +11,7 @@ def get_files(**kwargs):
     files = []
     for f in get_template_files(**kwargs):
         part = f.path.parts[0]
-        if part == 'my_app' or part == 'pyproject.toml' or part == 'README.md' or part == 'LICENSE.txt':
+        if part in {'my_app', 'pyproject.toml', 'README.md', 'LICENSE.txt'}:
             files.append(File(Path(relative_root, f.path), f.contents))
 
     files.append(
